"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = linter;

var _path = require("path");

var _fsExtra = require("fs-extra");

var _ESLintError = _interopRequireDefault(require("./ESLintError"));

var _getCLIEngine = _interopRequireDefault(require("./getCLIEngine"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function linter(options, compiler, callback) {
  try {
    const {
      CLIEngine,
      cli
    } = (0, _getCLIEngine.default)(options);
    const report = cli.executeOnFiles(options.files); // filter ignored files

    const results = report.results.filter(result => !cli.isPathIgnored(result.filePath)); // do not analyze if there are no results or eslint config

    if (!results || results.length < 1) {
      callback();
      return;
    } // if enabled, use eslint autofixing where possible


    if (options.fix) {
      CLIEngine.outputFixes(report);
    } // skip if no errors or warnings


    if (report.errorCount < 1 && report.warningCount < 1) {
      callback();
      return;
    }

    let {
      errors,
      warnings
    } = parseResults(options, results);
    compiler.hooks.afterEmit.tapAsync('ESLintWebpackPlugin', (compilation, next) => {
      if (warnings.length > 0) {
        compilation.warnings.push(new _ESLintError.default(options.formatter(warnings)));
        warnings = [];
      }

      if (errors.length > 0) {
        compilation.errors.push(new _ESLintError.default(options.formatter(errors)));
        errors = [];
      }

      next();
    });
    compiler.hooks.emit.tapAsync('ESLintWebpackPlugin', (compilation, next) => {
      const {
        outputReport
      } = options;

      if (!outputReport || !outputReport.filePath) {
        next();
        return;
      }

      const content = outputReport.formatter ? outputReport.formatter(results) : options.formatter(results);
      let {
        filePath
      } = outputReport;

      if (!(0, _path.isAbsolute)(filePath)) {
        filePath = (0, _path.join)(compilation.compiler.outputPath, filePath);
      }

      (0, _fsExtra.ensureFileSync)(filePath);
      (0, _fsExtra.writeFileSync)(filePath, content);
      next();
    });

    if (options.failOnError && errors.length > 0) {
      callback(new _ESLintError.default(options.formatter(errors)));
    } else if (options.failOnWarning && warnings.length > 0) {
      callback(new _ESLintError.default(options.formatter(warnings)));
    } else {
      callback();
    }
  } catch (e) {
    compiler.hooks.afterEmit.tapAsync('ESLintWebpackPlugin', (compilation, next) => {
      compilation.errors.push(new _ESLintError.default(e.message));
      next();
    });
    callback();
  }
}

function parseResults(options, results) {
  let errors = [];
  let warnings = [];

  if (options.emitError) {
    errors = results.filter(file => fileHasErrors(file) || fileHasWarnings(file));
  } else if (options.emitWarning) {
    warnings = results.filter(file => fileHasErrors(file) || fileHasWarnings(file));
  } else {
    warnings = results.filter(file => !fileHasErrors(file) && fileHasWarnings(file));
    errors = results.filter(fileHasErrors);
  }

  if (options.quiet && warnings.length > 0) {
    warnings = [];
  }

  return {
    errors,
    warnings
  };
}

function fileHasErrors(file) {
  return file.errorCount > 0;
}

function fileHasWarnings(file) {
  return file.warningCount > 0;
}